/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_SOURCE_atomic_c 
#define DEF_SOURCE_atomic_c "atomic.c"

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "mz.h"

static int __MZ_Lock__(int * volatile s_LockFlag, int s_Switch);

void MZ_AtomicAdd(int * volatile s_To, int s_Value);
void MZ_AtomicSub(int * volatile s_From, int s_Value);
int MZ_AtomicExchange(int * volatile s_To, int s_Value);
int MZ_InitLock(int * volatile s_Key);
void MZ_Lock(int * volatile s_Key);
void MZ_UnLock(int * volatile s_Key);

/* [ Static function ] ==================================================================== */
static int __MZ_Lock__(int * volatile s_LockFlag, int s_Switch)
{
 int s_Return;	
 int s_LockTime = 0, s_TimeOut;
 s_TimeOut = (s_Switch >> 2) * 100; 
L_SpinLoop:;
 s_Return = MZ_AtomicExchange(s_LockFlag, s_Switch); 
 if(s_Switch != 0 && s_Return != 0)
 {
  s_LockTime++;
  if(s_TimeOut != 0 && s_LockTime > s_TimeOut)
  {
   fprintf(stderr, "%s: %s - [ERROR] Dead-lock instead of off (%d second timeout) !!!\n", __FILE__, __FUNCTION__, s_TimeOut * 100);
   s_Return = MZ_AtomicExchange(s_LockFlag, 0); 
  } 
  else 
  {
#ifdef WIN32
   Sleep(10);
#else
   struct timeval s_LocalTimeVal = {(long)0, (long)10000};
   (void)select(0, (fd_set *)0, (fd_set *)0, (fd_set *)0, (struct timeval *)(&s_LocalTimeVal));
#endif
   goto L_SpinLoop; 
  }
 }
 return(s_Return);
}


/* [ Public function ] ==================================================================== */
void MZ_AtomicAdd(int * volatile s_To, int s_Value)
{
#ifdef WIN32
#else
 __asm__ volatile(
  "\n\t"
  "lock addl %1, (%0)\n\t"
  "\n\t"
  :
  : "r"(s_To), "r"(s_Value)
 );
#endif
}

void MZ_AtomicSub(int * volatile s_From, int s_Value)
{
#ifdef WIN32
#else
 __asm__ volatile(
  "\n\t"
  "lock subl %1, (%0)\n\t"
  "\n\t"
  :
  : "r"(s_From), "r"(s_Value)
 );
#endif
}

int MZ_AtomicExchange(int * volatile s_To, int s_Value)
{
 register int s_Return;
#ifdef WIN32
#else
 __asm__ volatile( 
  "\n\t"
  "lock xchgl %1, (%2)\n\t"
  "\n\t"
  : "=a"(s_Return)
  : "a"(s_Value), "r"(s_To)
 ); 
#endif
 return(s_Return);
}

int MZ_InitLock(int * volatile s_Key)
{
 if(s_Key)MZ_UnLock(s_Key);
 return(0);
}

void MZ_Lock(int * volatile s_Key)
{
 (void)__MZ_Lock__(s_Key, 1);
}

void MZ_UnLock(int * volatile s_Key)
{
 (void)__MZ_Lock__(s_Key, 0);
}

#endif

/* End of source */
