/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_SOURCE_dump_c 
#define DEF_SOURCE_dump_c "dump.c"

#ifndef DEF_STANDALONE 
#define DEF_STANDALONE (0)
#endif

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#if DEF_STANDALONE == (0)
#include "mz.h"
#else
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif

#ifndef DEF_MACRO_PEEK
#define DEF_MACRO_PEEK (1)
#define __MZ_PEEK__(s_From, s_Offset) (((unsigned char *)(s_From)) + (s_Offset))
#define __MZ_PEEK_R__(s_From, s_Offset) (((unsigned char *)(s_From)) - (s_Offset))
#endif

#define DEF_DUMP_WIDTH ( 16 )
#define DEF_DUMP_CACHE ( 1 << 10 )

static int __MZ_Dump_ConvertHexa__(void *s_String, int s_Value);
static void __MZ_Dump_Put__(const void *s_String, int s_Length);
static void __MZ_Dump_Hexa__(int s_Value, int s_Place);
static void __MZ_Dump__(int s_Offset, int s_Index, const void *s_Data, int s_Width);
void MZ_DumpEx(int s_Offset, const void *s_Data, int s_Size);
int MZ_DumpFile(const char *s_FileName, int s_Offset, int s_Size);
void MZ_Dump(const void *s_Data, int s_Size);

/* [ Static function ] ==================================================================== */

static int __MZ_Dump_ConvertHexa__(void *s_String, int s_Value)
{
 const char c_HexaTable[] = "0123456789ABCDEF";
 char s_FILO[8];
 int s_Return, s_Index;
 for(s_Return = 0;s_Value > 0;s_FILO[s_Return++] = c_HexaTable[s_Value & 0x0f], s_Value >>= 4);
 for(s_Index = s_Return;s_Index > 0;*(((char *)s_String)++) = s_FILO[--s_Index]); 
 *((char *)s_String) = '\0';
 return(s_Return);
}

static void __MZ_Dump_Put__(const void *s_String, int s_Length)
{
 static char sg_Cache[ DEF_DUMP_CACHE ];
 static int  sg_Index = 0;
#if DEF_STANDALONE == (0)
 static int  sg_Lock = 0;
 MZ_Lock(&sg_Lock);
#endif
 if((s_Length == 0) || ((sg_Index + s_Length) > sizeof(sg_Cache)))
 {
  (void)write(1, &sg_Cache[0], sg_Index);
  sg_Index = 0;
 }
 if(s_Length > 0)
 {
  if(s_Length >= sizeof(sg_Cache))(void)write(1, s_String, s_Length);
  else
  {
#if DEF_STANDALONE == (0) 
   MZ_CopyMemory((void *)(&sg_Cache[sg_Index]), s_String, s_Length);
#else
   memcpy((void *)(&sg_Cache[sg_Index]), s_String, s_Length);
#endif
   sg_Index += s_Length;
  }
 } 
#if DEF_STANDALONE == (0)
 MZ_UnLock(&sg_Lock);
#endif
}

static void __MZ_Dump_Hexa__(int s_Value, int s_Place)
{
 char s_Zero = '0', s_ConvertBuffer[8 + 1];
 int s_Length = __MZ_Dump_ConvertHexa__(&s_ConvertBuffer[0], s_Value), s_Index;
 s_Index = s_Length;
 while((s_Index++) < s_Place)__MZ_Dump_Put__(&s_Zero, sizeof(s_Zero));
 if(s_Length > 0)__MZ_Dump_Put__(&s_ConvertBuffer[0], s_Length);
}

static void __MZ_Dump__(int s_Offset, int s_Index, const void *s_Data, int s_Width)
{
 const char c_More[] = {' ', '|'}, c_Space = ' ', c_Dot = '.', c_NewLine = '\n';
 const char c_SpaceTable[] = {' ', ' ', ' ', ' ', ' '};	
 int s_Count;
 char s_Right[ 1 + 1 + DEF_DUMP_WIDTH + 1 + 1 ];	
 s_Right[0] = c_Space, s_Right[1] = '[', s_Right[ sizeof(s_Right) - 2 ] = ']', s_Right[ sizeof(s_Right) - 1 ] = c_NewLine;
 __MZ_Dump_Hexa__(s_Offset + s_Index, 8);
 for(s_Count = 0;s_Count < s_Width;s_Count++)
 {
  if(s_Count == (DEF_DUMP_WIDTH >> 1))__MZ_Dump_Put__(&c_More[0], sizeof(c_More));
  __MZ_Dump_Put__(&c_Space, sizeof(c_Space));
#if DEF_STANDALONE == (1)
  s_Right[ s_Count + 2 ] = *((char *)__MZ_PEEK__(s_Data, s_Count));
#else
  s_Right[ s_Count + 2 ] = MZ_PeekByte(s_Data, s_Count);
#endif
  __MZ_Dump_Hexa__(s_Right[ s_Count + 2 ], 2);

  /* Filter code */
  if(s_Right[ s_Count + 2 ] >= 0x7f || s_Right[ s_Count + 2 ] < c_Space)s_Right[ s_Count + 2 ] = c_Dot; 
 }
 for(;s_Count < DEF_DUMP_WIDTH;s_Count++)
 { 
  if(s_Count == (DEF_DUMP_WIDTH >> 1))__MZ_Dump_Put__(&c_SpaceTable[0], 2 + 3);
  else __MZ_Dump_Put__(&c_SpaceTable[0], 3);
  s_Right[ s_Count + 2 ] = c_Space; 
 } 
 __MZ_Dump_Put__(&s_Right[0], sizeof(s_Right));
}	

/* [ Public function ] ==================================================================== */
void MZ_DumpEx(int s_Offset, const void *s_Data, int s_Size)
{
 int s_Index = 0, s_Width;
 while(s_Index < s_Size)
 {
  s_Width = s_Size - s_Index; 
  if(s_Width > DEF_DUMP_WIDTH)s_Width = DEF_DUMP_WIDTH;
  __MZ_Dump__(s_Offset, s_Index, __MZ_PEEK__(s_Data, s_Index), s_Width);  
  s_Index += DEF_DUMP_WIDTH;
 }
 __MZ_Dump_Put__((void *)0, 0); 
}

int MZ_DumpFile(const char *s_FileName, int s_Offset, int s_Size)
{
 int s_Return, s_Handle, s_ReadBytes;
 unsigned char *s_Buffer[ DEF_DUMP_CACHE ];
 s_Return = (-1);
 if(s_FileName)
 {
  s_Handle = open(s_FileName, O_RDONLY);
  if(s_Handle >= 0)
  {
   if(s_Offset > 0)lseek(s_Handle, s_Offset, SEEK_SET);
   s_Return = 0;
   do
   {
    s_ReadBytes = read(s_Handle, &s_Buffer[0], sizeof(s_Buffer));
    if(s_ReadBytes > 0)
    {
     MZ_DumpEx((s_Offset > 0 ? s_Offset : 0) + s_Return, &s_Buffer[0], s_ReadBytes);
     s_Return += s_ReadBytes;
    }
   }while(s_ReadBytes == sizeof(s_Buffer) && (s_Size < 0 || s_Return < s_Size)); 
   close(s_Handle);
  }
 }
 return(s_Return);
}

void MZ_Dump(const void *s_Data, int s_Size)
{
 MZ_DumpEx(0, s_Data, s_Size);
}

#if DEF_STANDALONE == (1)
int main(int s_Argc, char *s_Argv[])
{
 int s_Return = 0, s_Index;
 
 if(s_Argc > 1)
 {
  s_Index = 1;
  while(s_Index < s_Argc)
  {
   fprintf(stdout, "Dump: \"%s\"\n"
                   "-----------------------------------------------------------------------------\n",
		   s_Argv[s_Index]);
   (void)MZ_DumpFile(s_Argv[s_Index], (-1), (-1));
   fprintf(stdout, "-----------------------------------------------------------------------------\n");
   s_Index++;
  }
 }
 else fprintf(stdout, "Usage: dump <file>...\n"); 
 
 return(s_Return);
}
#endif

#endif

/* End of source */
