/*
 Copyright (c) Information Equipment co.,LTD.
 All right reserved.
 
 Code by JaeHyuk Cho <mailto:minzkn@infoeq.com> Made in KOREA

 http://minzkn.pe.ky
*/

#ifndef DEF_SOURCE_md5_c 
#define DEF_SOURCE_md5_c "md5.c"

#ifndef DEF_STANDALONE
#define DEF_STANDALONE (0)
#endif

#if DEF_STANDALONE == (0)
#include "mz.h"
#else
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

typedef struct 
{
 unsigned long        State[4];
 unsigned long        Count[2];
 unsigned char        Buffer[64];
}t_MD5_Context;

#endif

static void MZ_MD5_Encode(unsigned char *s_Target, const unsigned long *s_Source, unsigned long s_Length);
static void MZ_MD5_Decode(unsigned long *s_Target, const unsigned char *s_Source, unsigned long s_Length);

void MZ_MD5_Init(t_MD5_Context *s_Context);
void MZ_MD5_Update(t_MD5_Context *s_Context, const void *s_Buffer, unsigned long s_Length);
void MZ_MD5_TransForm(void *s_Buffer, void *s_In);
void MZ_MD5_Final(t_MD5_Context *s_Context, void *s_Digest);

/* [ Static function ] ==================================================================== */
static void MZ_MD5_Encode(unsigned char *s_Target, const unsigned long *s_Source, unsigned long s_Length)
{
 unsigned long s_Count0, s_Count1;
 for(s_Count0 = 0, s_Count1 = 0;s_Count1 < s_Length;s_Count0++, s_Count1 += 4)
 {
  s_Target[s_Count1 + 0] = (unsigned char)(s_Source[s_Count0] & 0xff);
  s_Target[s_Count1 + 1] = (unsigned char)((s_Source[s_Count0] >> 8) & 0xff);
  s_Target[s_Count1 + 2] = (unsigned char)((s_Source[s_Count0] >> 16) & 0xff);
  s_Target[s_Count1 + 3] = (unsigned char)((s_Source[s_Count0] >> 24) & 0xff);
 }
}

static void MZ_MD5_Decode(unsigned long *s_Target, const unsigned char *s_Source, unsigned long s_Length)
{
 unsigned long s_Count0, s_Count1;
 for(s_Count0 = 0, s_Count1 = 0;s_Count1 < s_Length;s_Count0++, s_Count1 += 4)
 {
  s_Target[s_Count0] = (((unsigned long)s_Source[s_Count1 + 0])       ) |
                       (((unsigned long)s_Source[s_Count1 + 1]) << 8  ) |
                       (((unsigned long)s_Source[s_Count1 + 2]) << 16 ) |
                       (((unsigned long)s_Source[s_Count1 + 3]) << 24 ); 
 } 
}

/* [ Public function ] ==================================================================== */
void MZ_MD5_Init(t_MD5_Context *s_Context)
{
 s_Context->State[0] = 0x67452301lu;
 s_Context->State[1] = 0xefcdab89lu;
 s_Context->State[2] = 0x98badcfelu;
 s_Context->State[3] = 0x10325476lu;
 s_Context->Count[0] = s_Context->Count[1]  = 0lu;
}

void MZ_MD5_Update(t_MD5_Context *s_Context, const void *s_Buffer, unsigned long s_Length)
{
 unsigned long s_Count, s_Index, s_PartLen;
 s_Index = (unsigned long)((s_Context->Count[0] >> 3) & 0x3flu);
 s_Context->Count[0] += (s_Length << 3);
 if(s_Context->Count[0] < (s_Length << 3))s_Context->Count[1]++;
 s_Context->Count[1] += (s_Length >> 29);
 s_PartLen = 64 - s_Index; 
 if(s_Length >= s_PartLen)
 {
#if DEF_STANDALONE == (1)
  memcpy(&s_Context->Buffer[s_Index], s_Buffer, s_PartLen);
#else
  MZ_CopyMemory(&s_Context->Buffer[s_Index], s_Buffer, s_PartLen);
#endif
  MZ_MD5_TransForm(s_Context->State, s_Context->Buffer);
  for(s_Count = s_PartLen; s_Count + 63 < s_Length;s_Count += 64)MZ_MD5_TransForm(s_Context->State, ((unsigned char *)s_Buffer) + s_Count);
  s_Index = 0;	
 }
 else s_Count = 0;
#if DEF_STANDALONE == (1)
 memcpy(&s_Context->Buffer[s_Index], ((unsigned char *)s_Buffer) + s_Count, s_Length - s_Count);
#else
 MZ_CopyMemory(&s_Context->Buffer[s_Index], ((unsigned char *)s_Buffer) + s_Count, s_Length - s_Count);
#endif
}

#define S11 7
#define S12 12
#define S13 17
#define S14 22
#define S21 5
#define S22 9
#define S23 14
#define S24 20
#define S31 4
#define S32 11
#define S33 16
#define S34 23
#define S41 6
#define S42 10
#define S43 15
#define S44 21

#define DEF_MD5_F1(x,y,z) (((x) & (y)) | ((~x) & (z))) 
#define DEF_MD5_F2(x,y,z) (((x) & (z)) | ((y) & (~z))) 
#define DEF_MD5_F3(x,y,z) ((x) ^ (y) ^ (z))
#define DEF_MD5_F4(x,y,z) ((y) ^ ((x) | (~z)))
#define DEF_MD5_ROTATE_LEFT(x,n) (((x) << (n)) | ((x) >> (32 - (n))))

#define DEF_MD5_FF(a,b,c,d,x,s,ac) { \
	                              (a) += DEF_MD5_F1((b),(c),(d)) + (x) + (unsigned long)(ac); \
			              (a)  = DEF_MD5_ROTATE_LEFT((a),(s)); \
				      (a) += (b); \
				     } 
#define DEF_MD5_GG(a,b,c,d,x,s,ac) { \
	                              (a) += DEF_MD5_F2((b),(c),(d)) + (x) + (unsigned long)(ac); \
			              (a)  = DEF_MD5_ROTATE_LEFT((a),(s)); \
				      (a) += (b); \
				     } 
#define DEF_MD5_HH(a,b,c,d,x,s,ac) { \
	                              (a) += DEF_MD5_F3((b),(c),(d)) + (x) + (unsigned long)(ac); \
			              (a)  = DEF_MD5_ROTATE_LEFT((a),(s)); \
				      (a) += (b); \
				     } 
#define DEF_MD5_II(a,b,c,d,x,s,ac) { \
	                              (a) += DEF_MD5_F4((b),(c),(d)) + (x) + (unsigned long)(ac); \
			              (a)  = DEF_MD5_ROTATE_LEFT((a),(s)); \
				      (a) += (b); \
				     }

#define FF DEF_MD5_FF
#define GG DEF_MD5_GG
#define HH DEF_MD5_HH
#define II DEF_MD5_II

void MZ_MD5_TransForm(void *s_Buffer, void *s_In)
{
 unsigned long a, b, c, d;
 unsigned long s_In32[16];
 a = *(((unsigned long *)s_Buffer) + 0);
 b = *(((unsigned long *)s_Buffer) + 1);
 c = *(((unsigned long *)s_Buffer) + 2);
 d = *(((unsigned long *)s_Buffer) + 3);
 MZ_MD5_Decode(s_In32, s_In, 64);
 
 FF(a, b, c, d, s_In32[ 0], S11, 0xd76aa478); /* 1 */
 FF(d, a, b, c, s_In32[ 1], S12, 0xe8c7b756); /* 2 */
 FF(c, d, a, b, s_In32[ 2], S13, 0x242070db); /* 3 */
 FF(b, c, d, a, s_In32[ 3], S14, 0xc1bdceee); /* 4 */
 FF(a, b, c, d, s_In32[ 4], S11, 0xf57c0faf); /* 5 */
 FF(d, a, b, c, s_In32[ 5], S12, 0x4787c62a); /* 6 */
 FF(c, d, a, b, s_In32[ 6], S13, 0xa8304613); /* 7 */
 FF(b, c, d, a, s_In32[ 7], S14, 0xfd469501); /* 8 */
 FF(a, b, c, d, s_In32[ 8], S11, 0x698098d8); /* 9 */
 FF(d, a, b, c, s_In32[ 9], S12, 0x8b44f7af); /* 10 */
 FF(c, d, a, b, s_In32[10], S13, 0xffff5bb1); /* 11 */
 FF(b, c, d, a, s_In32[11], S14, 0x895cd7be); /* 12 */
 FF(a, b, c, d, s_In32[12], S11, 0x6b901122); /* 13 */
 FF(d, a, b, c, s_In32[13], S12, 0xfd987193); /* 14 */
 FF(c, d, a, b, s_In32[14], S13, 0xa679438e); /* 15 */
 FF(b, c, d, a, s_In32[15], S14, 0x49b40821); /* 16 */

 GG(a, b, c, d, s_In32[ 1], S21, 0xf61e2562); /* 17 */
 GG(d, a, b, c, s_In32[ 6], S22, 0xc040b340); /* 18 */
 GG(c, d, a, b, s_In32[11], S23, 0x265e5a51); /* 19 */
 GG(b, c, d, a, s_In32[ 0], S24, 0xe9b6c7aa); /* 20 */
 GG(a, b, c, d, s_In32[ 5], S21, 0xd62f105d); /* 21 */
 GG(d, a, b, c, s_In32[10], S22,  0x2441453); /* 22 */
 GG(c, d, a, b, s_In32[15], S23, 0xd8a1e681); /* 23 */
 GG(b, c, d, a, s_In32[ 4], S24, 0xe7d3fbc8); /* 24 */
 GG(a, b, c, d, s_In32[ 9], S21, 0x21e1cde6); /* 25 */
 GG(d, a, b, c, s_In32[14], S22, 0xc33707d6); /* 26 */
 GG(c, d, a, b, s_In32[ 3], S23, 0xf4d50d87); /* 27 */
 GG(b, c, d, a, s_In32[ 8], S24, 0x455a14ed); /* 28 */
 GG(a, b, c, d, s_In32[13], S21, 0xa9e3e905); /* 29 */
 GG(d, a, b, c, s_In32[ 2], S22, 0xfcefa3f8); /* 30 */
 GG(c, d, a, b, s_In32[ 7], S23, 0x676f02d9); /* 31 */
 GG(b, c, d, a, s_In32[12], S24, 0x8d2a4c8a); /* 32 */

 HH(a, b, c, d, s_In32[ 5], S31, 0xfffa3942); /* 33 */
 HH(d, a, b, c, s_In32[ 8], S32, 0x8771f681); /* 34 */
 HH(c, d, a, b, s_In32[11], S33, 0x6d9d6122); /* 35 */
 HH(b, c, d, a, s_In32[14], S34, 0xfde5380c); /* 36 */
 HH(a, b, c, d, s_In32[ 1], S31, 0xa4beea44); /* 37 */
 HH(d, a, b, c, s_In32[ 4], S32, 0x4bdecfa9); /* 38 */
 HH(c, d, a, b, s_In32[ 7], S33, 0xf6bb4b60); /* 39 */
 HH(b, c, d, a, s_In32[10], S34, 0xbebfbc70); /* 40 */
 HH(a, b, c, d, s_In32[13], S31, 0x289b7ec6); /* 41 */
 HH(d, a, b, c, s_In32[ 0], S32, 0xeaa127fa); /* 42 */
 HH(c, d, a, b, s_In32[ 3], S33, 0xd4ef3085); /* 43 */
 HH(b, c, d, a, s_In32[ 6], S34,  0x4881d05); /* 44 */
 HH(a, b, c, d, s_In32[ 9], S31, 0xd9d4d039); /* 45 */
 HH(d, a, b, c, s_In32[12], S32, 0xe6db99e5); /* 46 */
 HH(c, d, a, b, s_In32[15], S33, 0x1fa27cf8); /* 47 */
 HH(b, c, d, a, s_In32[ 2], S34, 0xc4ac5665); /* 48 */

 II(a, b, c, d, s_In32[ 0], S41, 0xf4292244); /* 49 */
 II(d, a, b, c, s_In32[ 7], S42, 0x432aff97); /* 50 */
 II(c, d, a, b, s_In32[14], S43, 0xab9423a7); /* 51 */
 II(b, c, d, a, s_In32[ 5], S44, 0xfc93a039); /* 52 */
 II(a, b, c, d, s_In32[12], S41, 0x655b59c3); /* 53 */
 II(d, a, b, c, s_In32[ 3], S42, 0x8f0ccc92); /* 54 */
 II(c, d, a, b, s_In32[10], S43, 0xffeff47d); /* 55 */
 II(b, c, d, a, s_In32[ 1], S44, 0x85845dd1); /* 56 */
 II(a, b, c, d, s_In32[ 8], S41, 0x6fa87e4f); /* 57 */
 II(d, a, b, c, s_In32[15], S42, 0xfe2ce6e0); /* 58 */
 II(c, d, a, b, s_In32[ 6], S43, 0xa3014314); /* 59 */
 II(b, c, d, a, s_In32[13], S44, 0x4e0811a1); /* 60 */
 II(a, b, c, d, s_In32[ 4], S41, 0xf7537e82); /* 61 */
 II(d, a, b, c, s_In32[11], S42, 0xbd3af235); /* 62 */
 II(c, d, a, b, s_In32[ 2], S43, 0x2ad7d2bb); /* 63 */
 II(b, c, d, a, s_In32[ 9], S44, 0xeb86d391); /* 64 */
 *(((unsigned long *)s_Buffer) + 0) += a;
 *(((unsigned long *)s_Buffer) + 1) += b;
 *(((unsigned long *)s_Buffer) + 2) += c;
 *(((unsigned long *)s_Buffer) + 3) += d;
#if DEF_STANDALONE == (1)
 memset(&s_In32[0], 0, sizeof(s_In32));
#else
 MZ_ZeroMemory(&s_In32[0], sizeof(s_In32));
#endif
}


void MZ_MD5_Final(t_MD5_Context *s_Context, void *s_Digest)
{
 unsigned char s_MD5_Padding[64] = {0x80, 0,};
 unsigned char s_Bits[8];
 unsigned long s_Index, s_PaddingLen;
 MZ_MD5_Encode(s_Bits, s_Context->Count, 8);
 s_Index = (unsigned long)((s_Context->Count[0] >> 3) & 0x3f);
 s_PaddingLen = (s_Index < 56) ? (56 - s_Index) : (120 - s_Index);
 MZ_MD5_Update(s_Context, s_MD5_Padding, s_PaddingLen);
 MZ_MD5_Update(s_Context, s_Bits, 8);
 MZ_MD5_Encode(s_Digest, s_Context->State, 16);
#if DEF_STANDALONE == (1)
 memset(s_Context, 0, sizeof(*s_Context));
#else
 MZ_ZeroMemory(s_Context, sizeof(*s_Context));
#endif 
}

#if DEF_STANDALONE == (1)
int main(int s_Argc, char *s_Argv[])
{
 int s_Return = 0, s_Index, s_Handle, s_ReadBytes;
 t_MD5_Context s_MD5;
 unsigned char s_Buffer[ 32 << 10 ];
 
 if(s_Argc > 1)
 {
  s_Index = 1;
  while(s_Index < s_Argc)
  {
   s_Handle = open(s_Argv[s_Index], O_RDONLY);
   if(s_Handle >= 0)
   {
    MZ_MD5_Init(&s_MD5);   
    do
    {
     s_ReadBytes = read(s_Handle, &s_Buffer[0], sizeof(s_Buffer));
     if(s_ReadBytes > 0)MZ_MD5_Update(&s_MD5, &s_Buffer[0], s_ReadBytes);
    }while(s_ReadBytes == sizeof(s_Buffer));
    close(s_Handle);
    MZ_MD5_Final(&s_MD5, &s_Buffer[0]);
    fprintf(stdout, "0x");
    for(s_ReadBytes = 0;s_ReadBytes < 16;s_ReadBytes++)fprintf(stdout, "%02X", s_Buffer[s_ReadBytes]);
    fprintf(stdout, " : %s\n", s_Argv[s_Index]);
   }
   else perror(s_Argv[s_Index]);
   s_Index++;
  }
 }
 else fprintf(stdout, "Usage: md5 <file>...\n"); 
 
 return(s_Return);
}
#endif

#endif

/* End of source */
