/*
 Copyright (c) 2002 Information Equipment co.,LTD.
 All Right Reserved.

 Code by JaeHyuk Cho <minzkn@infoeq.co.kr> 

 This example !!!
   - Simple is best.

*/

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <time.h>
#include <linux/fb.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

typedef struct ts_MZ_STAR
{
 struct ts_MZ_STAR *Next;
 int                x, y, Color, vx, vy, r, m;
 void (*DrawImage)(struct ts_MZ_STAR *);  
}t_MZ_STAR;

#define DEF_HEAT_CPU (100000)   /* 0 = Very busy check  ~  500000 = Slow check */
#define DEF_2PI (2.0 * 3.14159265358)

int                        g_FB_Handle, g_FB_Cx, g_FB_Cy, g_FB_ScreenSize;
struct fb_fix_screeninfo   g_FB_FIX;
struct fb_var_screeninfo   g_FB_VAR;
void                      *g_FB_Map;
void (*DrawPixel)(int, int, int);
t_MZ_STAR                 *g_MZ_STAR = (t_MZ_STAR *)0;

void DrawPixel8(int s_Color, int s_x, int s_y)
{
 *(((unsigned char *)g_FB_Map) + (s_y * g_FB_FIX.line_length) + s_x) = (unsigned char)s_Color;
}

void DrawPixel16(int s_Color, int s_x, int s_y)
{
 *((unsigned short *)(((unsigned char *)g_FB_Map) + (s_y * g_FB_FIX.line_length) + (s_x << 1))) = (unsigned short)s_Color;
}

void DrawPixel24(int s_Color, int s_x, int s_y)
{
 unsigned char *s_FB_Ptr = ((unsigned char *)g_FB_Map) + (s_y * g_FB_FIX.line_length) + ((s_x << 1) + s_x);
 *((unsigned short *)s_FB_Ptr) = (unsigned short)s_Color;
 *(s_FB_Ptr + sizeof(unsigned short)) = (unsigned char)(s_Color >> 16); 
}

void DrawPixel32(int s_Color, int s_x, int s_y)
{
 *((unsigned long *)(((unsigned char *)g_FB_Map) + (s_y * g_FB_FIX.line_length) + (s_x << 2))) = (unsigned long)s_Color;
}

void PinXY(int s_Angle, int *s_x, int *s_y, int s_r)
{ /* s_To(Grid) : X = s_From(Grid) : s_Current */
 double s_Value = ((DEF_2PI * (double)(s_Angle % 360)) / 360.0);
 *(s_x) = (int)(sin(s_Value) * s_r); *(s_y) = (int)(-cos(s_Value) * s_r); 
}

void __DrawLine__(int s_Color, float s_x1, float s_y1, float s_x2, float s_y2, int s_Level)
{ /* Call by call level 16 optimize draw line : JaeHyuk algorithm ^^ */
 float s_cx = (s_x1 + s_x2) / 2.0, s_cy = (s_y1 + s_y2) / 2.0;
 if(((int)s_x1 == (int)s_x2 && (int)s_y1 == (int)s_y2) || s_Level > 16)DrawPixel(s_Color, (int)s_cx, (int)s_cy);
 else
 {
  s_Level++;	
  __DrawLine__(s_Color, s_x1, s_y1, s_cx, s_cy, s_Level); __DrawLine__(s_Color, s_cx, s_cy, s_x2, s_y2, s_Level);
 }
}

void DrawLine(int s_Color, int s_x1, int s_y1, int s_x2, int s_y2)
{
 __DrawLine__(s_Color, (float)s_x1, (float)s_y1, (float)s_x2, (float)s_y2, 0);
}

void DrawCircle(int s_Color, int s_x, int s_y, int s_r)
{
 double s_Pi, s_Grid = 1.0 / ((double)s_r), s_sinX, s_cosY; 
 for(s_Pi = 0.0;s_Pi < (DEF_2PI / 4.0);s_Pi += s_Grid)
 {
  s_sinX = sin(s_Pi) * (double)s_r; s_cosY = -cos(s_Pi) * (double)s_r;	 
  DrawPixel(s_Color, (int)(s_x + s_sinX), (int)(s_y + s_cosY));
  DrawPixel(s_Color, (int)(s_x - s_sinX), (int)(s_y + s_cosY));
  DrawPixel(s_Color, (int)(s_x + s_sinX), (int)(s_y - s_cosY));
  DrawPixel(s_Color, (int)(s_x - s_sinX), (int)(s_y - s_cosY));
 }
}

void MZ_STAR_DrawImage0(struct ts_MZ_STAR *s_STAR)
{
 DrawCircle(0, s_STAR->x, s_STAR->y, s_STAR->r);
 s_STAR->x += s_STAR->vx;
 s_STAR->y += s_STAR->vy;
 if(s_STAR->x < s_STAR->r)
 {
  s_STAR->x = s_STAR->r;
  s_STAR->vx *= (-1);  
  s_STAR->m++;
 } 
 if(s_STAR->x >= (g_FB_VAR.xres - s_STAR->r))
 {
  s_STAR->x = (g_FB_VAR.xres - s_STAR->r) - 1;
  s_STAR->vx *= (-1);  
  s_STAR->m++;
 }
 if(s_STAR->y < s_STAR->r)
 {
  s_STAR->y = s_STAR->r;
  s_STAR->vy *= (-1);  
  s_STAR->m++;
 } 
 if(s_STAR->y >= (g_FB_VAR.yres - s_STAR->r))
 {
  s_STAR->y = (g_FB_VAR.yres - s_STAR->r) - 1;
  s_STAR->vy *= (-1);  
  s_STAR->m++;
 }
 DrawCircle(s_STAR->Color, s_STAR->x, s_STAR->y, s_STAR->r);
}

void MZ_STAR_ADD(void)
{
 t_MZ_STAR *s_NewSTAR;
 s_NewSTAR = (t_MZ_STAR *)malloc(sizeof(t_MZ_STAR));
 if(s_NewSTAR)
 {
  int s_Random;	 
  /* memset(s_NewSTAR, 0, sizeof(t_MZ_STAR)); */
  s_NewSTAR->Next = (t_MZ_STAR *)0;
  s_NewSTAR->m = 0;
  s_NewSTAR->x = g_FB_Cx;
  s_NewSTAR->y = g_FB_Cy;
  do
  {
   s_Random = (rand() % (16 + 1)) - 8;
   s_NewSTAR->vx = s_Random;
   s_Random = (rand() % (16 + 1)) - 8;
   s_NewSTAR->vy = s_Random;
  }while(s_NewSTAR->vx == 0 && s_NewSTAR->vy == 0);
  s_NewSTAR->r = (rand() % 10) + 1;
  do
  {
   s_NewSTAR->Color = rand() % 0xffff;	  
  }while(s_NewSTAR->Color == 0);
  /* s_NewSTAR->DrawImage = NULL;*/
  s_NewSTAR->DrawImage = MZ_STAR_DrawImage0;
  if(g_MZ_STAR)
  {
   s_NewSTAR->Next = g_MZ_STAR;
   g_MZ_STAR = s_NewSTAR;
  }
  else g_MZ_STAR = s_NewSTAR; 
 }
}

int main(int s_Argc, char *s_Argv[])
{
 char *s_Device = "/dev/fb0";	
 if(s_Argc > 1)s_Device = s_Argv[1];
 printf("fbstar v1.0.1 - Code by JaeHyuk Cho <minzkn@infoeq.co.kr>\n\n");
 g_FB_Handle = open(s_Device, O_RDWR);
 if(g_FB_Handle >= 0)
 {
  if(ioctl(g_FB_Handle, FBIOGET_FSCREENINFO, &g_FB_FIX) == 0)
  {
   if(ioctl(g_FB_Handle, FBIOGET_VSCREENINFO, &g_FB_VAR) == 0)
   {
    g_FB_Map = mmap((void *)0, g_FB_FIX.line_length * g_FB_VAR.yres, PROT_READ | PROT_WRITE, MAP_SHARED, g_FB_Handle, 0);    
    if(g_FB_Map != (void *)(-1))
    { 
     switch(g_FB_VAR.bits_per_pixel)
     {
      case  8: DrawPixel = DrawPixel8;  break;
      case 16: DrawPixel = DrawPixel16; break;
      case 24: DrawPixel = DrawPixel24; break;
      case 32: DrawPixel = DrawPixel32; break;
      default: DrawPixel = (void *)0;   break;
     }
     g_FB_Cx = g_FB_VAR.xres >> 1; g_FB_Cy = g_FB_VAR.yres >> 1;
     g_FB_ScreenSize = g_FB_VAR.xres < g_FB_VAR.yres ? g_FB_VAR.xres : g_FB_VAR.yres;
     if(DrawPixel)
     {
      t_MZ_STAR *s_STAR;	   
      int s_Count;
      srand((unsigned int)time((time_t *)0)); 
      
      for(s_Count = 0;s_Count < g_FB_Cy;s_Count++)MZ_STAR_ADD();

      do
      {
       if(g_MZ_STAR)
       {
        if(g_MZ_STAR->m > 0)
	{
         s_STAR = g_MZ_STAR;
         g_MZ_STAR = g_MZ_STAR->Next;
	 free(s_STAR);
	 MZ_STAR_ADD();
	}		
        s_STAR = g_MZ_STAR;
        while(s_STAR)
        {
	 if(s_STAR->DrawImage)s_STAR->DrawImage(s_STAR);	
         s_STAR = s_STAR->Next;
        }
       }
      }while(1);
     }
     else fprintf(stderr, "Err: %d\n", __LINE__);
     munmap(g_FB_Map, g_FB_FIX.line_length * g_FB_VAR.yres);
    }
    else fprintf(stderr, "Err: %d\n", __LINE__);
   }
   else fprintf(stderr, "Err: %d\n", __LINE__);
  }
  else fprintf(stderr, "Err: %d\n", __LINE__);
  close(g_FB_Handle);	 
 }
 else fprintf(stderr, "Err: %d\n", __LINE__);
 return(0);
}

/* End of source */ 
