/* 
  Copyright (c) 2004 JaeHyuk Cho <mailto:minzkn@infoeq.com>
  All right reserved.
*/

/*
  x86 16bit code library (Disk)
*/

/*
  - Interrupt $0x13
    + Function #0x02 : Read disk sectors
      %ah=$0x02
      %al=Number of sectors to read
      %ch=Track/cylinder number [7..0]
      %cl[5..0]=Sector number
      %cl[7..6]=Track/cylinder number [9..8]
      %dh=Head number
      %dl=Drive number (0=1st floppy, 1=2st floppy, 0x80=HDD0, 0x81=HDD1, ...)
      %es:%bx = Pointer to buffer
      <INT>
      %ah=Status return code
      %al=Number of sectors read
      Carry flag
        0=OK
	1=Error
      <DESCRIPTION>
      BIOS limit
        Number of sectors     : 1..128
        Track/cylinder number : 0..1023
        Head number           : 1..15
        Sector number         : 1..17
    + Function #0x08 : Get current disk parameters
      %ah=$0x08
      %dl=Drive number (0=1st floppy, 1=2st floppy, 0x80=HDD0, 0x81=HDD1, ...)
      <INT>
      %ah=Status return code
      %bl=CMOS drive type
        $0x01=5.25inch 360Kb
	$0x02=5.25inch 1.2Mb
	$0x03=3.5inch 720Kb
	$0x04=3.5inch 1.44Mb
      %ch=Tracks/cylinders [7..0]
      %cl[5..0]=Sectors per track/cylinder [5..0]
      %cl[7..6]=Tracks/cylinders [9..8]
      %dh=Heads (0 based)
      %dl=Number of drives attached
      %es:%di=Disk base table pointer (11byte)
      Carry flag
        $0=OK
	$1=Error
*/

P_ResetDisk:                 /* void P_ResetDisk(L_Drive) */
	xorb %ah, %ah
	movb (L_Drive), %dl
	int $0x13
	jnc 1f
	movw $0f, %si
	jmp L_Error
0:
	.string "Invalid disk !\r\n"
1:
	retw
L_Drive:
	.byte 0x00

P_GetDiskParameters:         /* L_Heads, L_SectorsPerCylinder P_GetDiskParameters(L_Drive) */
	movb $0x08, %ah
	movb (L_Drive), %dl
	int $0x13
	jnc 1f
	movw $0f, %si
	jmp L_Error
0:
	.string "Invalid disk !\r\n"
1:	
	movb %cl, (L_SectorsPerCylinder) 
	andb $0x3f, (L_SectorsPerCylinder) /* Sectors per cylinder : 6 bit */
	xchgb %ch, %cl
	shrb $6, %ch
	movw %cx, (L_Cylinders)
	incb %dh
	movb %dh, (L_Heads)
	retw

P_ReadDiskSector:            /* %ax near16 P_ReadDiskSector(L_Drive, L_Cylinder, L_Head, L_Sector) */
	movw $0x0201, %ax
	movw (L_Cylinder), %cx
	shlb $6, %ch
	orb (L_Sector), %ch
	xchgb %ch, %cl
	movb (L_Drive), %dl
	movb (L_Head), %dh
	int $0x13
	jnc 1f
	movw $0f, %si
	jmp L_Error
0:
	.string "Read error!\r\n"
1:	
	retw
L_Cylinder: /* Track/cylinder */
	.word 0x0000
L_Head:
	.byte 0x00
L_Sector:
	.byte 0x00

P_LBA_TO_CHS:                /* L_Cylinder, L_Head, L_Sector near16 P_LBA_TO_CHS(%dx:%ax, L_SectorsPerCylinder, L_Heads) */
	divw (L_SectorsPerCylinder)
	incb %dl
	movb %dl, (L_Sector)
	xorw %dx, %dx
	divw (L_Heads)
	movb %dl, (L_Head)
	movw %ax, (L_Cylinder)
	retw
L_SectorsPerCylinder: /* Sectors per track/cylinder */
	.word 0x0012 /* Default 18(0x0012) */
L_Cylinders:
	.word 0x0000
L_Heads:
	.word 0x0002 /* Default 2 */

P_LBA_ReadSector:            /* Carry flag near16 P_LBA_ReadSector(L_Drive, %dx:%ax, %es:%bx) */
	callw P_LBA_TO_CHS
	callw P_ReadDiskSector
	retw

/* End of source */
